@@ -18,6 +18,7 @@
  *
  * Authors:
  *      Mark McLoughlin <mark@skynet.ie>
+ *      Jonh Wendell <wendell@bani.com.br>
  *
  *
  *   The keyboard and pointer handling code is borrowed from
@@ -137,6 +138,7 @@
 
 #include <string.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtk.h>
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 #ifdef HAVE_XTEST
@@ -226,6 +228,8 @@ typedef struct
  */
 static VinoInputData global_input_data = { 0, };
 
+static gchar *cb_str = NULL;
+
 #ifdef HAVE_XTEST
 
 static struct {
@@ -487,7 +491,9 @@ vino_input_initialize_keycodes_core (Display *xdisplay)
 	      altgrsym = (keysyms_per_keycode <= VINO_LEVEL_ALTGR) ? NoSymbol :
 		XKeycodeToKeysym (xdisplay, keycode, VINO_LEVEL_ALTGR);
 
-	      unmodifiable = (shiftsym == NoSymbol) && (altgrsym == NoSymbol);
+	      unmodifiable =
+		(shiftsym == NoSymbol || shiftsym == sym) &&
+		(altgrsym == NoSymbol || altgrsym == sym);
 	    }
 
 	  if (unmodifiable)
@@ -759,12 +765,13 @@ vino_input_init (GdkDisplay *display)
       global_input_data.xtest_supported = TRUE;
     }
 
+  global_input_data.n_pointer_buttons = XGetPointerMapping (xdisplay, NULL, 0);
+
 #ifdef HAVE_XKB
   if (XkbQueryExtension (xdisplay, NULL, &global_input_data.xkb_event_type,
 			 NULL, NULL, NULL))
     {
       XkbStateRec state;
-  global_input_data.n_pointer_buttons = XGetPointerMapping(xdisplay, NULL, 0);
 
       dprintf (INPUT, "Using XKB\n");
 
@@ -1110,9 +1117,45 @@ vino_input_handle_key_event (GdkScreen *screen,
 #endif /* HAVE_XTEST */
 }
 
-void
-vino_input_handle_clipboard_event (GdkScreen *screen,
-				   char      *text,
-				   int        len)
+/* text was actually requested */
+static void
+copy_cb (GtkClipboard     *clipboard,
+         GtkSelectionData *data,
+	 guint             info,
+	 VinoServer       *server)
 {
+  gtk_selection_data_set_text (data, cb_str, -1);
+}
+
+void
+vino_input_handle_clipboard_event (GdkScreen  *screen,
+				   char       *text,
+				   int         len,
+				   VinoServer *server)
+{
+  GtkClipboard *cb;
+  gsize a, b;
+  GtkTargetEntry targets[] = {
+				{"UTF8_STRING", 0, 0},
+				{"COMPOUND_TEXT", 0, 0},
+				{"TEXT", 0, 0},
+				{"STRING", 0, 0},
+			     };
+
+  if (!text)
+    return;
+
+  g_free (cb_str);
+  cb_str = g_convert (text, len, "utf-8", "iso8859-1", &a, &b, NULL);
+  if (cb_str)
+    {
+      cb = gtk_clipboard_get_for_display (gdk_screen_get_display (screen),
+                                          GDK_SELECTION_CLIPBOARD);
+      gtk_clipboard_set_with_owner (cb,
+				    targets,
+				    G_N_ELEMENTS(targets),
+				    (GtkClipboardGetFunc) copy_cb,
+				    NULL,
+				    G_OBJECT (server));
+    }
 }
